package net.wanji.web.kafka;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.gts.GeometryUtil;
import net.wanji.common.gts.GeomsConvertUtil;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.vo.LockControlVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.common.constant.Constant;
import net.wanji.web.mapper.SpecialServiceCrossMapper;
import net.wanji.web.mapper.SpecialServiceMapper;
import net.wanji.web.po.SpecialServicePO;
import net.wanji.web.websocket.WebSocketServer;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author duanruiming
 * @date 2023/03/09 16:03
 */
@Slf4j
@Component
public class ConsumerHandler implements KafkaListenerErrorHandler {
    private final SpecialServiceMapper specialServiceMapper;
    private final SpecialServiceCrossMapper specialServiceCrossMapper;
    private final UtcFeignClients utcFeignClients;
    public static String carPlate;
    public static List<BaseCrossInfoPO> crossList = new ArrayList<>();

    public ConsumerHandler(SpecialServiceMapper specialServiceMapper, SpecialServiceCrossMapper specialServiceCrossMapper, BaseCrossInfoMapper baseCrossInfoMapper, @Qualifier("net.wanji.feign.service.UtcFeignClients") UtcFeignClients utcFeignClients) {
        this.specialServiceMapper = specialServiceMapper;
        this.specialServiceCrossMapper = specialServiceCrossMapper;
        this.utcFeignClients = utcFeignClients;
    }

    public static void updateCarplate(String newCarPlate) {
        carPlate = newCarPlate;
    }

    public static void updateCrossList(List<BaseCrossInfoPO> newCrossList) {
        crossList = newCrossList;
    }

    /**
     * kafka数据缓存
     * key:topic value:data
     */
    private static final Map<String, JNMatchResultMiniData> carInfoMap = new ConcurrentHashMap<>();

    /**
     * 通过消息主题获取消息信息
     *
     * @param topic
     * @return
     */
    public JNMatchResultMiniData getTopicMessage(String topic) {
        if (carInfoMap.isEmpty()) {
            return null;
        }
        return carInfoMap.get(topic);
    }

    @KafkaListener(topics = {"CSMatchResultMiniData_0"})
    public void receiveCarInfoData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) {
        try {
            log.debug("收到kafka消息，消息主题：" + record.topic());
            log.debug("收到kafka消息，消息内容：" + record.value());
            JNMatchResultMiniData jnMatchResultMiniData = JSONObject.parseObject((String)record.value(), JNMatchResultMiniData.class);
            carInfoMap.put(record.topic(), jnMatchResultMiniData);
            acknowledgment.acknowledge();
            // 特勤车辆经过自动解锁路口
            List<JNMatchResultMiniData.E1FrameParticipant> carList = jnMatchResultMiniData.getE1FrameParticipant();
            for (JNMatchResultMiniData.E1FrameParticipant car : carList) {
                String picLicense = car.getPicLicense();
                // 是否是特勤车辆
                if (StrUtil.equals(picLicense, carPlate)) {
                    // 判断是否经过自动解锁路口
                    for (BaseCrossInfoPO cross : crossList) {
                        // 车辆位置
                        double carLong = car.getLongitude();
                        double carLat = car.getLatitude();
                        String carLonLatStr = carLong + "," + carLat;
                        Geometry carGeo = GeometryUtil.str2Geometry(carLonLatStr);
                        // 路口缓冲区
                        String crossLocation = cross.getLocation();
                        double[] crossLonLat = CrossUtil.getLonLat(crossLocation);
                        String crossLonLatStr = crossLonLat[0] + "," + crossLonLat[1];
                        Geometry crossGeo = GeometryUtil.str2Geometry(crossLonLatStr);
                        Geometry crossBuff = GeomsConvertUtil.getGeometryBuf(crossGeo, "80.0");
                        if (carGeo.intersects(crossBuff)) {
                            // 解锁
                            LockControlVO vo = new LockControlVO();
                            String crossId = cross.getId();
                            vo.setCrossCode(crossId);
                            vo.setCommand(0);
                            vo.setDuration(999);
                            vo.setPhaseList(new ArrayList<>());
                            utcFeignClients.lockControl(vo);
                            // 修改路口为非自动解锁路口，状态为已解锁
                            SpecialServicePO specialServicePO = specialServiceMapper.selectIdByPlateNum(picLicense);
                            Integer specialServiceId = specialServicePO.getId();
                            specialServiceCrossMapper.updateStatusFail(crossId, specialServiceId);
                            // 给前端推送WebSocket
                            Set<WebSocketServer> events =
                                    WebSocketServer.getWebSocketSet(Constant.WEBSOCKET_AUTO_UNLOCK);
                            if (CollectionUtil.isNotEmpty(events)) {
                                for (WebSocketServer socketServer : events) {
                                    socketServer.sendMessage(crossId);
                                }
                            }
                            crossList.remove(cross);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("监听主题消息异常， 异常信息", e);
        }
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

}
