package net.wanji.web.task;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.vo.SignalStatusVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.cache.BaseCrossInfoCache;
import net.wanji.web.common.constant.Constant;
import net.wanji.web.common.constant.ControlModeConst;
import net.wanji.web.entity.TCrossControlHist;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.entity.TDeviceStatusLog;
import net.wanji.web.mapper.ControlHistMapper;
import net.wanji.web.mapper.TDeviceStatusLogMapper;
import net.wanji.web.mapper.TDeviceStatusMapper;
import net.wanji.web.vo.situationDetection.CommonEventAlarmOutVo;
import net.wanji.web.websocket.WebSocketServer;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @author duanruiming
 * @date 2023/01/03 10:43
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class SignalStatusTask {

    /**
     * todo 当前有使用crossId,control路口编号,系统名称作为用户编号，导致出现相同用户登录会把之前websocket连接抢占，导致原来websocket
     * 不推送问题
     *  1、通过统一认证，一个用户只能登录一次，其他连接不需要推送
     *  2、通过将crossId路口编号作为topic，推送灯态时发送给多有相同的主题
     */

    private final UtcFeignClients utcFeignClients;
    private final TDeviceStatusMapper tDeviceStatusMapper;
    private final ControlHistMapper controlHistMapper;
    private final TDeviceStatusLogMapper tDeviceStatusLogMapper;
    private final BaseCrossInfoCache baseCrossInfoCache;

    @Scheduled(fixedRate = 30 * 1000)
    @Transactional
    public void syncSignalStatus() {
        try {
            long start = LocalDateTime.now().toEpochSecond(ZoneOffset.of("+8"));
            log.info("===当前时间：{}", LocalDateTime.now());
            JsonViewObject jsonViewObject = utcFeignClients.runningStatusAlarm();
            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
                log.error("定时任务同步信号机设备状态utcService调用异常");
                return;
            }
            // utc服务信号机状态列表
            ObjectMapper mapper = JacksonUtils.getInstance();
            List<SignalStatusVO> content = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<SignalStatusVO>>() {
            });
            // 信号机状态信息
            List<BaseCrossInfoPO> baseCrossInfoPOS = baseCrossInfoCache.getCrossInfoCache();
            List<CommonEventAlarmOutVo> results = new ArrayList<>();
            content.forEach(signalStatusVO -> {
                String crossId = signalStatusVO.getCrossId();
                String name = getSignalName(baseCrossInfoPOS, crossId);
                Integer currentSignalStatus = signalStatusVO.getStatus() == null ? 0 : signalStatusVO.getStatus();
                Integer currentFaultType = signalStatusVO.getFaultType() == null ? 0 : signalStatusVO.getFaultType();
                // 信号机在线，但是故障，数据库状态为2
                if (currentSignalStatus == 1 && !Objects.equals(0, currentFaultType)) {
                    currentSignalStatus = 2;
                }
                // 更新数据库状态
                updateSignalStatus(signalStatusVO, crossId, name, currentSignalStatus, currentFaultType, results);
                insertControlHist(signalStatusVO);
                log.info("更新信号机状态成功，{}", currentSignalStatus);
            });
            long end = LocalDateTime.now().toEpochSecond(ZoneOffset.of("+8"));
            log.info("===定时任务消费时间：{}s", end - start);

            // 无告警数据不推送
            if (!CollectionUtils.isEmpty(results)) {
                JsonViewObject warnData = JsonViewObject.newInstance();
                LocalDateTime now = LocalDateTime.now();
                warnData.setTimestamp(now.toInstant(ZoneOffset.of("+8")).toEpochMilli());
                warnData.setContent(results);
                WebSocketServer.sendTopicInfo2All(JSONObject.toJSONString(warnData), Constant.WEBSOCKET_TOPIC_EVENT_WARN);
            }
        } catch (Exception e) {
            log.error("定时任务同步信号机设备状态远程utcService调用异常", e);
        }
    }

    private static String getSignalName(List<BaseCrossInfoPO> finalBaseCrossInfoPOList, String crossId) {
        String name = null;
        for (BaseCrossInfoPO baseCrossInfoPO : finalBaseCrossInfoPOList) {
            if (StringUtils.equals(crossId, baseCrossInfoPO.getId())) {
                name = baseCrossInfoPO.getName();
                break;
            }
        }
        return name;
    }

    /**
     * 更新信号机状态
     *
     * @param signalStatusVO
     * @param crossId
     * @param name
     * @param currentSignalStatus
     * @param currentFaultType
     */
    private void updateSignalStatus(SignalStatusVO signalStatusVO, String crossId, String name, Integer currentSignalStatus, Integer currentFaultType, List<CommonEventAlarmOutVo> results) {
        LambdaQueryWrapper<TDeviceStatusInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(crossId)) {
            lambdaQueryWrapper.eq(TDeviceStatusInfo::getCode, crossId);
            TDeviceStatusInfo tDeviceStatusInfo = tDeviceStatusMapper.selectOne(lambdaQueryWrapper);
            if (Objects.nonNull(tDeviceStatusInfo)) {
                if (isExecuteUpdate(currentSignalStatus, currentFaultType, tDeviceStatusInfo)) {
                    tDeviceStatusInfo.setStatus(currentSignalStatus);
                    tDeviceStatusInfo.setFaultType(currentFaultType);
                    tDeviceStatusInfo.setGmtCreate(new Date()); // 实时数据，创建和修改时间一致
                    tDeviceStatusMapper.updateById(tDeviceStatusInfo);
                    insertDeviceStatusLog(signalStatusVO, crossId, currentSignalStatus, currentFaultType, name);
                    // 信号机状态发生改变，进行告警通知
                    buildSignalWarnData(results, crossId, currentSignalStatus, currentFaultType, name);
                }
            } else {
                TDeviceStatusInfo deviceStatusInfo = new TDeviceStatusInfo();
                deviceStatusInfo.setCode(crossId);
                deviceStatusInfo.setName(name);
                deviceStatusInfo.setType(1);
                deviceStatusInfo.setFaultType(currentFaultType);
                deviceStatusInfo.setStatus(currentSignalStatus);
                deviceStatusInfo.setGmtCreate(new Date());
                deviceStatusInfo.setId(null);
                tDeviceStatusMapper.insert(deviceStatusInfo);
            }
        }
    }

    /**
     * 插入控制历史表
     *
     * @param signalStatusVO
     */
    private void insertControlHist(SignalStatusVO signalStatusVO) {
        // 手动控制控制历史表
        Map<String, Boolean> centerControlMap = ControlModeConst.centerControlMap;
        if (centerControlMap.isEmpty() || (!centerControlMap.isEmpty() && !centerControlMap.get(signalStatusVO.getCrossId()))) {
            String crossId = signalStatusVO.getCrossId();
            Integer controlType = signalStatusVO.getControlType();
            TCrossControlHist tCrossControlHist = controlHistMapper.selectRecentOne(crossId);
            if (Objects.nonNull(tCrossControlHist) ) {
                if (!Objects.equals(controlType, tCrossControlHist.getType())) {
                    if (Objects.equals(1, controlType)) {
                        controlHistMapper.insertOne(crossId, 1, "现场手动");
                    } else if (!Objects.equals(7, tCrossControlHist.getType())){
                        controlHistMapper.insertOne(crossId, 7, "现场手动");
                    }
                }
            }
        }
    }

    /**
     * 信号机状态插入设备日志表
     *
     * @param signalStatusVO
     * @param crossId
     * @param currentSignalStatus
     * @param currentFaultType
     */
    private void insertDeviceStatusLog(SignalStatusVO signalStatusVO, String crossId, Integer currentSignalStatus, Integer currentFaultType, String name) {
        // 插入日志表
        TDeviceStatusLog tDeviceStatusLogInfoDB = tDeviceStatusLogMapper.selectLastOne(crossId);
        if (Objects.nonNull(tDeviceStatusLogInfoDB) && tDeviceStatusLogInfoDB.getFaultType() != currentFaultType) {
            tDeviceStatusLogInfoDB.setStatus(currentSignalStatus);
            tDeviceStatusLogInfoDB.setFaultType(tDeviceStatusLogInfoDB.getFaultType());
            tDeviceStatusLogInfoDB.setGmtCreate(new Date());
            tDeviceStatusLogInfoDB.setId(null);
            tDeviceStatusLogMapper.insert(tDeviceStatusLogInfoDB);
        } else {
            TDeviceStatusLog tDeviceStatusLog = new TDeviceStatusLog();
            tDeviceStatusLog.setCode(crossId);
            tDeviceStatusLog.setName(name);
            tDeviceStatusLog.setType(1);
            tDeviceStatusLog.setStatus(signalStatusVO.getStatus());
            tDeviceStatusLog.setFaultType(signalStatusVO.getFaultType());
            tDeviceStatusLog.setGmtCreate(new Date());
            tDeviceStatusLog.setId(null);
            tDeviceStatusLogMapper.insert(tDeviceStatusLog);
        }
    }

    public static boolean isExecuteUpdate(Integer currentSignalStatus, Integer currentFaultType, TDeviceStatusInfo tDeviceStatusInfo) {
        // 故障自定义，当故障时，状态对应海康在线
        int status = tDeviceStatusInfo.getStatus() == 2 ? 1 : tDeviceStatusInfo.getStatus();
        return currentSignalStatus != status || currentFaultType != tDeviceStatusInfo.getFaultType();
    }

    /**
     * 构建告警推送数据，放入results统一推送
     *
     * @param results
     * @param crossId
     * @param currentStatus
     * @param currentFaultType
     * @param crossName
     */
    private void buildSignalWarnData(List<CommonEventAlarmOutVo> results, String crossId, Integer currentStatus, Integer currentFaultType, String crossName) {
        try {
            CommonEventAlarmOutVo commonEventAlarmOutVo = new CommonEventAlarmOutVo();
            commonEventAlarmOutVo.setEventType(3);
            commonEventAlarmOutVo.setEventId(crossId);
            commonEventAlarmOutVo.setEventName(crossName);
            if (currentStatus == 0) {
                commonEventAlarmOutVo.setStatus("11");
                commonEventAlarmOutVo.setEventDesc(crossName.concat("信号机离线"));
            } else if (currentStatus == 1) {
                commonEventAlarmOutVo.setStatus(String.valueOf(0));
                commonEventAlarmOutVo.setEventDesc(crossName.concat("信号机上线"));
            } else {
                commonEventAlarmOutVo.setStatus(String.valueOf(currentFaultType));
                commonEventAlarmOutVo.setEventDesc(crossName.concat("信号机故障"));
            }
            LocalDateTime now = LocalDateTime.now();
            String formatNow = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(now);
            commonEventAlarmOutVo.setStartTime(formatNow);
            String coordinate = baseCrossInfoCache.getCoordinateByCrossId(crossId);
            commonEventAlarmOutVo.setCoordinate(coordinate);
            results.add(commonEventAlarmOutVo);
        } catch (Exception e) {
            log.error("构建信号机告警数据失败", e);
        }
    }
}
