package net.wanji.web.websocket;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author duanruiming
 * @date 2023/01/06 8:58
 */
@ServerEndpoint("/websocket/{topic}/{userId}")
@Component
@Slf4j
public class WebSocketServer {

    /**
     * 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
     */
    private static int onlineCount = 0;
    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     */
    private static final ConcurrentHashMap<String, Set<WebSocketServer>> webSocketMap = new ConcurrentHashMap<>();
    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session;
    /**
     * 主题
     */
    private String topic = "";
    /**
     * 接收userId
     */
    private String userId = "";

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("topic") String topic, @PathParam("userId") String userId) {
        this.session = session;
        this.topic = topic;
        this.userId = userId;
        int currentTopicSize = 0;
        if (webSocketMap.containsKey(topic)) {
            Set<WebSocketServer> set = webSocketMap.get(topic);
            set.add(this);
            webSocketMap.put(topic, set);
            addOnlineCount();
            currentTopicSize = set.size();
            //加入set中
        } else {
            Set<WebSocketServer> set = new HashSet<>();
            set.add(this);
            webSocketMap.put(topic, set);
            //加入set中
            currentTopicSize = set.size();
            addOnlineCount();
            //在线数加1
        }
        log.info("当前主题:{}, socket连接:{}, 当前主题连接数量:{}, 所有socket连接数量:{}", topic, userId, currentTopicSize, getOnlineCount());
        try {
            sendMessage("连接成功");
        } catch (IOException e) {
            log.error("主题：{}， socket:{}, 网络异常！", topic, userId);
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        if (webSocketMap.containsKey(topic)) {
            webSocketMap.remove(topic);
            //从set中删除
            subOnlineCount();
            log.info("{}主题连接关闭, 当前剩余连接数量：{}", topic, getOnlineCount());
        }
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(@PathParam("topic") String topic, @PathParam("userId") String userId, String message, Session session) {
        log.info("当前主题:{}, socket连接:{}, 消息:{}", topic, userId, message);
        //可以群发消息
        //消息保存到数据库、redis
        if (StringUtils.isNotBlank(topic) && StringUtils.isNotBlank(message)) {
            try {
                //解析发送的报文
                JSONObject jsonObject = JSON.parseObject(message);
                //追加发送人(防止串改)
                jsonObject.put("fromUserId", this.userId);
                String toUserId = jsonObject.getString("toUserId");
                //传送给对应toUserId用户的websocket
                if (webSocketMap.containsKey(topic)) {
                    Set<WebSocketServer> sets = webSocketMap.get(topic);
                    for (WebSocketServer socket : sets) {
                        String currentUserId = socket.getUserId();
                        if (StringUtils.equals(currentUserId, userId)) {
                            socket.sendMessage(message);
                            break;
                        }
                    }
                }
            } catch (Exception e) {
                log.error("主题:{}, socket:{}, 发送消息失败, 异常信息:{}", topic, userId, e);
            }
        }
    }

    /**
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("主题:{}，socket:{}，连接异常，异常信息:{}", topic, userId, error.getMessage());
    }

    /**
     * 实现服务器主动推送
     */
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
    }


    /**
     * 发送自定义消息
     */
    public static void sendInfo(String message, String topic, String userId) throws IOException {
        if (StringUtils.isNotBlank(topic) && webSocketMap.containsKey(topic)) {
            try {
                Set<WebSocketServer> socketSets = webSocketMap.get(topic);
                for (WebSocketServer socketSet : socketSets) {
                    String currentUserId = socketSet.getUserId();
                    if (StringUtils.equals(currentUserId, userId)) {
                        socketSet.sendMessage(message);
                        break;
                    }
                }
            } catch (Exception e) {
                log.error("主题:{}, socket:{}, 发送消息异常, 异常信息:{}", topic, userId, e);
            }
        } else {
            log.error("当前主题不存在:{}", topic);
        }
    }

    /**
     * 发送消息给所有连接
     *
     * @param message
     * @param topic
     * @throws IOException
     */
    public static void sendTopicInfo2All(String message, String topic) throws IOException {
        if (StringUtils.isNotBlank(topic) && webSocketMap.containsKey(topic)) {
            try {
                Set<WebSocketServer> socketSets = webSocketMap.get(topic);
                for (WebSocketServer socketSet : socketSets) {
                    socketSet.sendMessage(message);
                }
            } catch (Exception e) {
                log.error("主题:{}, 发送消息异常, 异常信息:{}", topic, e);
            }
        } else {
            log.error("当前主题不存在:{}", topic);
        }
    }

    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        WebSocketServer.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        WebSocketServer.onlineCount--;
    }

    public String getUserId() {
        return userId;
    }

    public String getTopic() {
        return topic;
    }

    public static Set<WebSocketServer> getWebSocketSet(String topic) {
        if (!webSocketMap.isEmpty()) {
            return webSocketMap.get(topic);
        }
        return Collections.EMPTY_SET;
    }
}
